#include <stdint.h>

#define P8(x)	(*(volatile uint8_t *)(x))
#define P16(x)	(*(volatile uint16_t *)(x))
#define P32(x)	(*(volatile uint32_t *)(x))

/* Clock Generation Circuit */

#define SCKCR	P32 (0x80020)

/* Power control */

#define MSTPCRA P32 (0x80010)
#define MSTPCRB P32 (0x80014)
#define MSTPCRC P32 (0x80018)

/* Serial Communications Interface */

typedef struct {
  uint8_t smr;
  uint8_t brr;
  uint8_t scr;
  uint8_t tdr;
  uint8_t ssr;
  uint8_t rdr;
  uint8_t scmr;
  uint8_t semr;
} SCI_Registers;

#define SCI	((volatile SCI_Registers *)(0x88240))

/* Timer Pulse Units */

typedef struct {
  uint8_t tcr;
  uint8_t tmdr;
  uint8_t tiorh;
  uint8_t tiorl;
  uint8_t tier;
  uint8_t tsr;
  uint16_t tcnt;
  uint16_t tgra;
  uint16_t tgrb;
  uint16_t tgrc;
  uint16_t tgrd;
} TPU_Registers;

#define TPU	((volatile TPU_Registers *)(0x88110))
#define TSTRA	P8 (0x88100)
#define TSYRA	P8 (0x88101)
#define TSTRB	P8 (0x88170)
#define TSYRB	P8 (0x88171)

/* Port I/O Control */

#define DDR	((volatile uint8_t *)0x8c000)
#define DR	((volatile uint8_t *)0x8c020)
#define PORT	((volatile uint8_t *)0x8c040)
#define ICR	((volatile uint8_t *)0x8c060)
#define PFCR0	P8 (0x8c100)
#define PFCR1	P8 (0x8c101)
#define PFCR2	P8 (0x8c102)
#define PFCR3	P8 (0x8c103)
#define PFCR4	P8 (0x8c104)
#define PFCR5	P8 (0x8c105)
#define PFCR6	P8 (0x8c106)
#define PFCR7	P8 (0x8c107)
#define PFCR8	P8 (0x8c108)
#define PFCR9	P8 (0x8c109)

/* DAC */

#define DADR0	P16 (0x880c0)
#define DADR1	P16 (0x880c2)
#define DACR	P8  (0x880c4)
#define DADPR	P8  (0x880c5)

/* ADC */

typedef struct {
  uint16_t addra;
  uint16_t addrb;
  uint16_t addrc;
  uint16_t addrd;
  char _filler1[8];
  uint8_t adcsr;
  uint8_t adcr;
  uint8_t addpr;
  uint8_t adsstr;
  char _filler2[12];
} ADC_Registers;

#define ADC	((volatile ADC_Registers *)(0x88040))


/* Interrupts */

#define IER0D	P8 (0x8720D)
#define IER1B	P8 (0x8721B)
#define IER1C	P8 (0x8721C)
#define IER1D	P8 (0x8721D)

#define IPR4C	P8 (0x8734C)
#define IPR4D	P8 (0x8734D)
#define IPR4E	P8 (0x8734E)
#define IPR81	P8 (0x87381)
#define IPR84	P8 (0x87384)

typedef void (*Interrupt_Handler)(void);
extern Interrupt_Handler _vectors[256];

#define IVEC_TPU0_TGI0A 104
#define IVEC_TPU0_TCI0V	108
#define IVEC_TPU1_TGI1A 111

#define IVEC_SCI4_RXI4	231
#define IVEC_SCI4_TXI4	232
#define IVEC_SCI4_TEI4	233

#define printf iprintf
