#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <ctype.h>
#include <usb.h>

#define WRITE_EP	0x01
#define READ_EP		0x82

static usb_dev_handle *handle;

main (int argc, char **argv)
{
  int read_mode = 0;
  int write_mode = 0;
  char *filename;
  FILE *f;
  char buf[64];
  int o;
  struct usb_bus *bus;
  struct usb_device *dev;
  int count;

  while ((o = getopt (argc, argv, "rw")) != -1)
    switch (o)
      {
      case 'r':
	read_mode = 1;
	break;
      case 'w':
	write_mode = 1;
	break;
      case '?':
	fprintf(stderr, "Usage: usbtool [-r] [-w] filename\n");
	exit (1);
      }

  if (!argv[optind])
    filename = 0;
  else
    filename = argv[optind];

  usb_init ();
  usb_find_busses ();
  count = usb_find_devices ();

  for (bus = usb_get_busses(); bus; bus=bus->next)
    {

      for (dev = bus->devices; dev; dev=dev->next)
	{
	  if (dev->descriptor.idVendor == 0x045b
	      && dev->descriptor.idProduct == 0x0025)
	    {
	      goto found_it;
	    }
	}
    }

  printf("No RX USB devices found.\n");
  exit (1);

 found_it:

  handle = usb_open (dev);

  usb_claim_interface (handle, 0);

  if (write_mode)
    {
      f = filename ? fopen (filename, "rb") : stdin;
      if (!f)
	{
	  fprintf (stderr, "Can't read %s\n", argv[optind]);
	  exit (1);
	}

      while (1)
	{
	  int w;
	  int r = fread (buf, 1, 64, f);
	  if (r <= 0)
	    break;
	  w = usb_bulk_write (handle, WRITE_EP, (char *)buf, r, 10000);
	  if (w != r)
	    {
	      fprintf (stderr, "write error, wrote %d out of %d bytes\n", w, r);
	      exit (1);
	    }
	}
    }

  if (read_mode)
    {
      f = filename ? fopen (filename, "wb") : stdout;
      if (!f)
	{
	  fprintf (stderr, "Can't write %s\n", argv[optind]);
	  exit (1);
	}

      while (1)
	{
	  int w, r;

	  r = usb_bulk_read (handle, READ_EP, (char *)buf, 64, 10000);
	  if (r == 0)
	    continue;
	  if (r < 0)
	    {
	      printf("usb_bulk_read returns %d\n", r);
	      break;
	    }
	  w = fwrite (buf, 1, r, f);
	  fflush (f);
	  if (w != r)
	    {
	      fprintf (stderr, "write error, wrote %d out of %d bytes\n", w, r);
	      exit (1);
	    }
	}
    }

  usb_release_interface (handle, 0);
  usb_close (handle);
  fclose (f);

  return 0;
}
